package java.sql;

import java.util.ServiceLoader;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Iterator;
import sun.reflect.CallerSensitive;
import sun.reflect.Reflection;
import java.util.Properties;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.util.concurrent.CopyOnWriteArrayList;

public class DriverManager
{
    private static final CopyOnWriteArrayList<DriverInfo> registeredDrivers;
    private static volatile int loginTimeout;
    private static volatile PrintWriter logWriter;
    private static volatile PrintStream logStream;
    private static final Object logSync;
    static final SQLPermission SET_LOG_PERMISSION;
    static final SQLPermission DEREGISTER_DRIVER_PERMISSION;
    
    private DriverManager() {
    }
    
    public static PrintWriter getLogWriter() {
        return DriverManager.logWriter;
    }
    
    public static void setLogWriter(PrintWriter logWriter) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(DriverManager.SET_LOG_PERMISSION);
        }
        DriverManager.logStream = null;
        DriverManager.logWriter = logWriter;
    }
    
    @CallerSensitive
    public static Connection getConnection(String s, Properties properties) throws SQLException {
        return getConnection(s, properties, Reflection.getCallerClass());
    }
    
    @CallerSensitive
    public static Connection getConnection(String s, String value, String value2) throws SQLException {
        Properties properties = new Properties();
        if (value != null) {
            properties.put("user", value);
        }
        if (value2 != null) {
            properties.put("password", value2);
        }
        return getConnection(s, properties, Reflection.getCallerClass());
    }
    
    @CallerSensitive
    public static Connection getConnection(String s) throws SQLException {
        return getConnection(s, new Properties(), Reflection.getCallerClass());
    }
    
    @CallerSensitive
    public static Driver getDriver(String str) throws SQLException {
        println("DriverManager.getDriver(\"" + str + "\")");
        Class<Object> callerClass = Reflection.getCallerClass();
        for (DriverInfo driverInfo : DriverManager.registeredDrivers) {
            if (isDriverAllowed(driverInfo.driver, callerClass)) {
                try {
                    if (driverInfo.driver.acceptsURL(str)) {
                        println("getDriver returning " + driverInfo.driver.getClass().getName());
                        return driverInfo.driver;
                    }
                    continue;
                }
                catch (SQLException ex) {}
            }
            else {
                println("    skipping: " + driverInfo.driver.getClass().getName());
            }
        }
        println("getDriver: no suitable driver");
        throw new SQLException("No suitable driver", "08001");
    }
    
    public static synchronized void registerDriver(Driver driver) throws SQLException {
        registerDriver(driver, null);
    }
    
    public static synchronized void registerDriver(Driver obj, DriverAction driverAction) throws SQLException {
        if (obj != null) {
            DriverManager.registeredDrivers.addIfAbsent(new DriverInfo(obj, driverAction));
            println("registerDriver: " + obj);
            return;
        }
        throw new NullPointerException();
    }
    
    @CallerSensitive
    public static synchronized void deregisterDriver(Driver obj) throws SQLException {
        if (obj == null) {
            return;
        }
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(DriverManager.DEREGISTER_DRIVER_PERMISSION);
        }
        println("DriverManager.deregisterDriver: " + obj);
        DriverInfo o = new DriverInfo(obj, null);
        if (DriverManager.registeredDrivers.contains(o)) {
            if (isDriverAllowed(obj, Reflection.getCallerClass())) {
                DriverInfo driverInfo = (DriverInfo)DriverManager.registeredDrivers.get(DriverManager.registeredDrivers.indexOf(o));
                if (driverInfo.action() != null) {
                    driverInfo.action().deregister();
                }
                DriverManager.registeredDrivers.remove(o);
            }
            else {
                throw new SecurityException();
            }
        }
        else {
            println("    couldn't find driver to unload");
        }
        return;
    }
    
    @CallerSensitive
    public static Enumeration<Driver> getDrivers() {
        Vector<DriverInfo> vector = new Vector();
        Class<Object> callerClass = Reflection.getCallerClass();
        for (DriverInfo driverInfo : DriverManager.registeredDrivers) {
            if (isDriverAllowed(driverInfo.driver, callerClass)) {
                vector.addElement(driverInfo.driver);
            }
            else {
                println("    skipping: " + driverInfo.getClass().getName());
            }
        }
        return vector.elements();
    }
    
    public static void setLoginTimeout(int loginTimeout) {
        DriverManager.loginTimeout = loginTimeout;
    }
    
    public static int getLoginTimeout() {
        return DriverManager.loginTimeout;
    }
    
    @Deprecated
    public static void setLogStream(PrintStream printStream) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(DriverManager.SET_LOG_PERMISSION);
        }
        if ((DriverManager.logStream = printStream) != null) {
            DriverManager.logWriter = new PrintWriter(printStream);
        }
        else {
            DriverManager.logWriter = null;
        }
    }
    
    @Deprecated
    public static PrintStream getLogStream() {
        return DriverManager.logStream;
    }
    
    public static void println(String x) {
        synchronized (DriverManager.logSync) {
            if (DriverManager.logWriter != null) {
                DriverManager.logWriter.println(x);
                DriverManager.logWriter.flush();
            }
        }
    }
    
    private static boolean isDriverAllowed(Driver driver, Class<?> clazz) {
        return isDriverAllowed(driver, (clazz != null) ? clazz.getClassLoader() : null);
    }
    
    private static boolean isDriverAllowed(Driver driver, ClassLoader loader) {
        boolean n = false;
        if (driver != null) {
            Class<Object> forName = null;
            try {
                forName = Class.forName(driver.getClass().getName(), true, loader);
            }
            catch (Exception ex) {}
            n = (forName == driver.getClass());
        }
        return n;
    }
    
    private static void loadInitialDrivers() {
        String str;
        try {
            str = (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
                @Override
                public String run() {
                    return System.getProperty("jdbc.drivers");
                }
            });
        }
        catch (Exception ex) {
            str = null;
        }
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                Iterator<Driver> iterator = ServiceLoader.load(Driver.class).iterator();
                try {
                    while (iterator.hasNext()) {
                        iterator.next();
                    }
                }
                catch (Throwable t) {}
                return null;
            }
        });
        println("DriverManager.initialize: jdbc.drivers = " + str);
        if (str == null || str.equals("")) {
            return;
        }
        String[] split = str.split(":");
        println("number of Drivers:" + split.length);
        for (String s : split) {
            try {
                println("DriverManager.Initialize: loading " + s);
                Class.forName(s, true, ClassLoader.getSystemClassLoader());
            }
            catch (Exception obj) {
                println("DriverManager.Initialize: load failed: " + obj);
            }
        }
        return;
    }
    
    private static Connection getConnection(String str, Properties properties, Class<?> clazz) throws SQLException {
        ClassLoader contextClassLoader = (clazz != null) ? clazz.getClassLoader() : null;
        synchronized (DriverManager.class) {
            if (contextClassLoader == null) {
                contextClassLoader = Thread.currentThread().getContextClassLoader();
            }
        }
        if (str == null) {
            throw new SQLException("The url cannot be null", "08001");
        }
        println("DriverManager.getConnection(\"" + str + "\")");
        SQLException obj = null;
        for (DriverInfo driverInfo : DriverManager.registeredDrivers) {
            if (isDriverAllowed(driverInfo.driver, contextClassLoader)) {
                try {
                    println("    trying " + driverInfo.driver.getClass().getName());
                    Connection connect = driverInfo.driver.connect(str, properties);
                    if (connect != null) {
                        println("getConnection returning " + driverInfo.driver.getClass().getName());
                        return connect;
                    }
                    continue;
                }
                catch (SQLException ex) {
                    if (obj == null) {
                        obj = ex;
                    }
                    else {
                        continue;
                    }
                }
            }
            else {
                println("    skipping: " + driverInfo.getClass().getName());
            }
        }
        if (obj != null) {
            println("getConnection failed: " + obj);
            throw obj;
        }
        println("getConnection: no suitable driver found for " + str);
        throw new SQLException("No suitable driver found for " + str, "08001");
    }
    
    static {
        registeredDrivers = new CopyOnWriteArrayList();
        DriverManager.loginTimeout = 0;
        DriverManager.logWriter = null;
        DriverManager.logStream = null;
        logSync = new Object();
        loadInitialDrivers();
        println("JDBC DriverManager initialized");
        SET_LOG_PERMISSION = new SQLPermission("setLog");
        DEREGISTER_DRIVER_PERMISSION = new SQLPermission("deregisterDriver");
    }
}
